BOOST = 65
BITE_SIZE = 5
LOOK_DIST =100
boosted = false
moving = false

Creature = {}
Creature.__index = Creature

function Creature.create(xpos,ypos,image, world)
	local c = {}
	setmetatable(c, Creature)
	c.energy = 20
	c.food = 20
	c.health = 20
	c.age = 0
	c.sleeping = false
	c.dead = false
	c.size = 1
	c.speed = 20
	c.pos = {x = xpos, y = ypos}
	c.rotation = 0
	c.image = image
	
	c.body = love.physics.newBody(world, xpos, ypos, "dynamic")
	return c
end

function Creature:Eat(food, dt)
	bite = BITE_SIZE * self.size * dt
	eaten = 0
	if food ~= nil then
		if food.amount ~= nil then
			if food.amount < bite then
				eaten = food.amount
			else
				eaten = bite
			end
			food.amount = food.amount - eaten
		
		elseif food.growth ~= nil then
			-- Eating a seed
			amount = food.health
			if amount < bite then
				eaten = amount
			else
				eaten = bite
			end
			food.health = food.health-eaten
			if food.health <= 0 then
				self.target = nil
			end
			
		elseif food.food ~= nil then
			-- CANNABLISM!
			if food.size * 20 < bite then
				eaten = food.size* 20
			else
				eaten = bite
			end
			food.size = food.size - eaten/20
			if food.size <= 0 then
				self.target = nil
			end
		end
	self.food = self.food + eaten
	
	end
end

function Creature:FindFood()
	tgt = nil
	dist = 999
	for _, f in ipairs(food) do
		if f.amount > 0 then
			d = distance(f.pos, self.pos)
			if d < (LOOK_DIST*self.size) and d <(math.random(0.5, 1.1)*dist) then
				tgt = f
				dist = d
			end
		end
	end
	
	if tgt == nil then
		for _, p in ipairs(plants) do
			if  p.stage < 4 and p.health > 0 then
				d = distance(p.pos, self.pos)
				if d < (LOOK_DIST*self.size) and d < (math.random(0.5, 1.1)*dist) then
					tgt = p
					dist = d
				end
			end
		end
	end
	
	if tgt == nil then
		for _, c in ipairs(creatures) do
			if c.dead then
				d = distance(f.pos, self.pos)
				if d < (LOOK_DIST*self.size) and (d < (math.random(0.5, 1.1)*dist)) then
					tgt = c
					dist = d
				end
			end
		end
	end
	
	self.target = tgt
end

function distance(p1, p2)
	return math.abs(p1.x-p2.x) + math.abs(p1.y-p2.y)
end

function love.load()
	lighting, scene = love.graphics.newCanvas(800,600), love.graphics.newCanvas(800,600)
	lights =  love.graphics.newCanvas(800,600)
	
	--grass1 = love.graphics.newQuad(0, 352, 32, 32, 672, 736)
	--grass2 = love.graphics.newQuad(32, 352, 32, 32, 672, 736)
	--grass3 = love.graphics.newQuad(64, 352, 32, 32, 672, 736)
	world = love.physics.newWorld()
	local_time = 6
	day_length = 600
	hour_length = day_length/24
	
	img = {
		monster = love.graphics.newImage("unit1.png"),
		bug_red = love.graphics.newImage("bug_red.png"),
		bug_yellow = love.graphics.newImage("bug_yellow.png"),
		bug_blue = love.graphics.newImage("bug_blue.png"),
		bug_dead = love.graphics.newImage("bug_dead.png"),
		food_large = love.graphics.newImage("food_large.png"),
		food_medium = love.graphics.newImage("food_medium.png"),
		food_small = love.graphics.newImage("food_small.png"),
		--terrain = love.graphics.newImage("terrain.png"),
		wheat_simple = {
					love.graphics.newImage("plants/wheat_simple/wheat_01.png"),
					love.graphics.newImage("plants/wheat_simple/wheat_02.png"),
					love.graphics.newImage("plants/wheat_simple/wheat_03.png"),
					love.graphics.newImage("plants/wheat_simple/wheat_04.png"),
					love.graphics.newImage("plants/wheat_simple/wheat_05.png"),
		},
		day = {
			afternoon = love.graphics.newImage("day_parts/afternoon.png"),
			late_afternoon = love.graphics.newImage("day_parts/late_afternoon.png"),
			early_evening = love.graphics.newImage("day_parts/early_evening.png"),
			evening = love.graphics.newImage("day_parts/evening.png"),
			night = love.graphics.newImage("day_parts/night.png"),
			early_morning = love.graphics.newImage("day_parts/early_morning.png"),
			morning = love.graphics.newImage("day_parts/morning.png"),
		},
		light = love.graphics.newImage("light_blur.png"),
		paused = love.graphics.newImage("paused.png")
	}
	
	creatures = {
		Creature.create(math.random(100,600), math.random(100,600), img.bug_red, world),
		Creature.create(math.random(100,600), math.random(100,600), img.bug_yellow, world),
		Creature.create(math.random(100,600), math.random(100,600), img.bug_blue, world)
	}
	
	plants = {}
	for i = 0, math.random(5,30) do
		plants[i] = {
			pos = { x = math.random(10, 800), y = math.random(10, 500) },	
			growth = 0,
			speed = math.random(1,4),
			health = math.random(15,40),
			flipped = (math.random(1, 10) > 4),
			rotation = math.random(0,0.4),
			stage = 1
		}
	end
	paused = false
	
	food = {}
	for i = 1, math.random(3,10) do
		food[i] = {pos = { x = math.random(40, 500), y = math.random(40, 500)}, amount = math.random(5,38)}
	end
	units = {
		{
			pos = { x = math.random(200,600), y = math.random(200,400) },
			speed = 100,
			moveDir = { 0, 0 }
		}
	}
	player = units[1]
	player.body = love.physics.newBody(world, player.pos.x, player.pos.y, "dynamic")
	player.shape = love.physics.newRectangleShape(64,64)
	player.fixture = love.physics.newFixture(player.body, player.shape)
	
	--ground = { 	body = love.physics.newBody(world, 300,300,"static"), 
		--		shape = love.physics.newRectangleShape(600,600)
			--}
	--ground.fixture = love.physics.newFixture(ground.body, ground.shape)
	
	love.graphics.setBackgroundColor(47,129,54)
end

function love.update(dt)
	if not paused then
	m = units[1]
	m.pos.x = m.pos.x + (m.moveDir[1]*m.speed * dt)
	m.pos.y = m.pos.y + (m.moveDir[2]*m.speed * dt) 
	
	for _, c in ipairs(creatures) do
		if not c.dead then
			if c.health <= 0 then
				c.dead = true
			else
				if c.food >70 then
					c.size = c.size + (0.025*dt)
				end
				if c.target == nil then
					--if c.food < 80 then
						c:FindFood()
					--end
					
					if c.target == nil then -- Couldn't find food
												
						dist = (c.speed*dt)
										
						if c.update ~= nil then
							c.update = c.update - dt
						end
						
						if c.update == nil or c.update < 0 then
							c.xdir = math.random(1,3)-2
							c.ydir = math.random(1,3)-2
							
							c.update = 6
						end
						
						c.update = c.update - dt
						newx = c.pos.x + (dist*c.xdir)
						if newx > 0 and newx < 800 then
							c.pos.x = newx
						end
						newy  = c.pos.y + (dist*c.ydir)
						if newy > 0 and newy < 600 then
							c.pos.y = newy
						end
						--c.pos.y= c.pos.y+(c.speed*dt)
						
					end
				else
					if distance(c.pos, c.target.pos) < 36 then
						f = c.target
						c:Eat(f, dt)
						if f.amount ~= nil then
							if f.amount == 0  then
								c.target = nil
							end
						elseif f.health ~= nil then
							if f.health <= 0 then
								c.target = nil
							end
						end
					else
						--a = math.atan2((c.pos.y-c.target[2]), (c.pos.x - c.target[1]))
						a = math.atan2((c.pos.x-c.target.pos.x), (c.target.pos.y-c.pos.y))
						if c.rotation ~= a then
							amt = dt*2
							diff = c.rotation - a
							--if math.abs(diff) < amt then
								c.rotation = a
							--else
							--	c.rotation = c.rotation - amt
								--if c.rotation < 0 then
								--	c.rotation = c.rotation + (2*math.pi)
								--end
									
								
							--end
						end
						xdist = c.speed * dt * c.size
						xdir = 1
						if math.abs(c.target.pos.x-c.pos.x) < xdist then 
							xdist = c.pos.x-c.target.pos.x
						end
						if c.target.pos.x < c.pos.x then
							xdir = -1
						end
						ydist = c.speed * dt * c.size
						ydir = 1
						if math.abs(c.target.pos.y-c.pos.y) < ydist then 
							ydist = c.pos.y-c.target.pos.y
						end
						if c.target.pos.y < c.pos.y then
							ydir = -1
						end
						
						c.pos.x = c.pos.x + (xdist*xdir)
						c.pos.y= c.pos.y + (ydist*ydir)
					end
				end
				if c.food <= 0 then 
					c.health = c.health - dt
				else
					c.food = c.food - dt
				end
			end
		end
	end
	
	for _, p in ipairs(plants) do
		if p.growth < 100 and p.health > 10 and (math.random(1,10) + (p.growth/25)) > 8 then
			p.growth = p.growth + (p.speed*dt)
			if p.growth > 100 then p.growth = 100 end
			p.stage = math.ceil(p.growth/10)
		end
	end
	
	
	local_time = (local_time + (dt/hour_length)) % 24
	world:update(dt)
	end
end

function love.draw()

	love.graphics.setCanvas(scene)
	love.graphics.clear()
	--love.graphics.drawq(img.terrain, grass1, 145, 256)
	--love.graphics.drawq(img.terrain, grass2, 400, 70)
	--love.graphics.drawq(img.terrain, grass3, 50, 200)
	--love.graphics.drawq(img.terrain, grass1, 500, 340)
	--love.graphics.drawq(img.terrain, grass2, 300, 240)
	
	for _, f in ipairs(food) do	
		if f.amount > 0 then
			if f.amount > 30 then
				s = 1.25 --((amt-65) / 25)+0.5
				i = img.food_large
			elseif f.amount >15 then
				s = 1--((amt-30) / 25)+0.5
				i = img.food_medium
			else
				s = 1 --((amt) / 25)+0.5
				i = img.food_small
			end
			love.graphics.draw(i, f.pos.x, f.pos.y, 0, s, s, 16, 16)
		end
	end
	
	
	for _, c in pairs(creatures) do
		if c.size > 0 then
			if c.dead then
				--love.graphics.draw(img.bug_dead, c.pos.x, c.pos.y, c.rotation, c.size,c.size, 16, 16)
				love.graphics.draw(img.bug_dead, c.pos.x, c.pos.y, c.rotation, 1,1, 16, 16)
			else
				--love.graphics.draw(c.image, c.pos.x, c.pos.y, c.rotation, c.size,c.size,16,16)
				love.graphics.draw(c.image, c.pos.x, c.pos.y, c.rotation, 1,1,16,16)
			end
		end
	end
	
	love.graphics.draw(img.monster, units[1].pos.x-16, units[1].pos.y-16)
	
	for _, p in ipairs(plants) do
		pimg = img.wheat_simple[math.floor(p.stage/2)]
		if pimg ~= nil and p.health > 0 then
			if p.flipped then
				love.graphics.draw(pimg, p.pos.x-4, p.pos.y-30)	
			else
				love.graphics.draw(pimg, p.pos.x-4, p.pos.y-30)	
			end
		end
	end
	
	if false then
	if local_time < 6 and local_time > 4 then
		love.graphics.draw(img.day.early_morning)
	elseif local_time > 6 and local_time < 8 then
		love.graphics.draw(img.day.morning)
	elseif local_time > 16 and local_time < 17 then
		love.graphics.draw(img.day.afternoon)
	elseif local_time > 17 and local_time < 18 then
		love.graphics.draw(img.day.late_afternoon)
	elseif local_time > 18 and local_time < 19 then
		love.graphics.draw(img.day.evening)
	elseif local_time > 19 or local_time < 4 then
		love.graphics.draw(img.day.night)
	end
	end
	
	
	-- draw light
	lighting:clear()
	love.graphics.setCanvas(lighting)
	love.graphics.setColor(0,0,0,80)
	love.graphics.rectangle('fill',0,0, love.graphics.getWidth(), love.graphics.getHeight())
	love.graphics.setColor(255,255,255,255)
	love.graphics.circle('fill', player.pos.x,player.pos.y,100,20)
	
	--love.graphics.draw(img.light, player.pos.x-64, player.pos.y-64)
	
	-- multiply canvas
	love.graphics.setCanvas()
	love.graphics.draw(lighting)
	love.graphics.setBlendMode('multiplicative')
	love.graphics.draw(scene)
	love.graphics.setBlendMode('alpha')
	
	if paused then
		love.graphics.draw(img.paused, 10, 540)
	end
	
	love.graphics.print(math.floor(local_time), 0,0)
end

function love.keypressed(key, unicode)
	if key == "up" or key == "down" or key == "left" or key == "right" then
		moving = true
	end
	
	if key == "up" then
		units[1].moveDir[2] = -1
	elseif key == "down" then
		units[1].moveDir[2] = 1
	elseif key == "right" then
		units[1].moveDir[1] = 1
	elseif key == "left" then
		units[1].moveDir[1] = -1
	end
end

function love.keyreleased(key, unicode)
	
	
	if key == "up" then
		units[1].moveDir[2] = 0
	elseif key == "down" then
		units[1].moveDir[2] = 0
	elseif key == "right" then
		units[1].moveDir[1] = 0
	elseif key == "left" then
		units[1].moveDir[1] = 0
	elseif key == "f" then
		food[#food+1] = {pos = { x = units[1].pos.x, y = units[1].pos.y }, amount = math.random(5,38)} 
	elseif key == "p" then
		plants[#plants+1] = {
			pos = { x = units[1].pos.x, y = units[1].pos.y },	
			growth = 0,
			speed = math.random(3,4),
			health = math.random(10, 100),
			flipped = (math.random(1, 10) > 4),
			rotation = math.random(0,0.4),
			stage = 1
		}
	elseif key == " " then
		paused = not paused
	end
	
	if key == "up" or key == "down" or key == "left" or key == "right" then
		moving = (units[1].moveDir[1] ~= 0) or (units[1].moveDir[2] ~= 0)
		if boosted and not moving then
			
			units[1].speed = units[1].speed - BOOST
			boosted = false
		end
	end
end